  $ dune build @runtest @install-file
          rbin alias runtest
  Cppome
  hello world
  Bar
  Foo
  lib: [
    "_build/install/default/lib/rlib/META" {"META"}
    "_build/install/default/lib/rlib/bar.mli" {"bar.mli"}
    "_build/install/default/lib/rlib/bar.re" {"bar.re"}
    "_build/install/default/lib/rlib/cppome.re" {"cppome.re"}
    "_build/install/default/lib/rlib/cppome.rei" {"cppome.rei"}
    "_build/install/default/lib/rlib/foo.ml" {"foo.ml"}
    "_build/install/default/lib/rlib/foo.rei" {"foo.rei"}
    "_build/install/default/lib/rlib/hello.re" {"hello.re"}
    "_build/install/default/lib/rlib/hello.rei" {"hello.rei"}
    "_build/install/default/lib/rlib/opam" {"opam"}
    "_build/install/default/lib/rlib/pped.re" {"pped.re"}
    "_build/install/default/lib/rlib/pped.rei" {"pped.rei"}
    "_build/install/default/lib/rlib/rlib$ext_lib" {"rlib$ext_lib"}
    "_build/install/default/lib/rlib/rlib.cma" {"rlib.cma"}
    "_build/install/default/lib/rlib/rlib.cmi" {"rlib.cmi"}
    "_build/install/default/lib/rlib/rlib.cmt" {"rlib.cmt"}
    "_build/install/default/lib/rlib/rlib.cmx" {"rlib.cmx"}
    "_build/install/default/lib/rlib/rlib.cmxa" {"rlib.cmxa"}
    "_build/install/default/lib/rlib/rlib.cmxs" {"rlib.cmxs"}
    "_build/install/default/lib/rlib/rlib.dune" {"rlib.dune"}
    "_build/install/default/lib/rlib/rlib.ml" {"rlib.ml"}
    "_build/install/default/lib/rlib/rlib__Bar.cmi" {"rlib__Bar.cmi"}
    "_build/install/default/lib/rlib/rlib__Bar.cmt" {"rlib__Bar.cmt"}
    "_build/install/default/lib/rlib/rlib__Bar.cmti" {"rlib__Bar.cmti"}
    "_build/install/default/lib/rlib/rlib__Bar.cmx" {"rlib__Bar.cmx"}
    "_build/install/default/lib/rlib/rlib__Cppome.cmi" {"rlib__Cppome.cmi"}
    "_build/install/default/lib/rlib/rlib__Cppome.cmt" {"rlib__Cppome.cmt"}
    "_build/install/default/lib/rlib/rlib__Cppome.cmti" {"rlib__Cppome.cmti"}
    "_build/install/default/lib/rlib/rlib__Cppome.cmx" {"rlib__Cppome.cmx"}
    "_build/install/default/lib/rlib/rlib__Foo.cmi" {"rlib__Foo.cmi"}
    "_build/install/default/lib/rlib/rlib__Foo.cmt" {"rlib__Foo.cmt"}
    "_build/install/default/lib/rlib/rlib__Foo.cmti" {"rlib__Foo.cmti"}
    "_build/install/default/lib/rlib/rlib__Foo.cmx" {"rlib__Foo.cmx"}
    "_build/install/default/lib/rlib/rlib__Hello.cmi" {"rlib__Hello.cmi"}
    "_build/install/default/lib/rlib/rlib__Hello.cmt" {"rlib__Hello.cmt"}
    "_build/install/default/lib/rlib/rlib__Hello.cmti" {"rlib__Hello.cmti"}
    "_build/install/default/lib/rlib/rlib__Hello.cmx" {"rlib__Hello.cmx"}
    "_build/install/default/lib/rlib/rlib__Pped.cmi" {"rlib__Pped.cmi"}
    "_build/install/default/lib/rlib/rlib__Pped.cmt" {"rlib__Pped.cmt"}
    "_build/install/default/lib/rlib/rlib__Pped.cmti" {"rlib__Pped.cmti"}
    "_build/install/default/lib/rlib/rlib__Pped.cmx" {"rlib__Pped.cmx"}
  ]
  bin: [
    "_build/install/default/bin/refmt" {"refmt"}
  ]
