Reproduction case for #1529: using an extension when no dune-project
file is present.

  $ dune build
  Info: creating file dune-project with this contents: (lang dune 1.6)
  Info: appending this line to dune-project: (using menhir 2.0)
