  $ jbuilder build @foo
  File "src/dune", line 4, characters 10-17:
  4 |  (c_names stubs/x))
                ^^^^^^^
  Error: File src/stubs/x.c is not part of the current directory group. This is not allowed.
  [1]
